@@ -16,3 +16,15 @@ class InvalidGalleryAttachments(APIException):
 class BadCredentials(APIException):
     status_code = status.HTTP_400_BAD_REQUEST
     default_detail = 'Invalid user credentials'
+
+class PollClosed(APIException):
+    status_code = status.HTTP_400_BAD_REQUEST
+    default_detail = 'Poll closed'
+
+class InvalidPoll(APIException):
+    status_code = status.HTTP_400_BAD_REQUEST
+    default_detail = 'Invalid poll'
+
+class UnpermittedActionError(APIException):
+    status_code = status.HTTP_401_UNAUTHORIZED
+    default_detail = 'You do not have permission to perform this action'
