@@ -1,27 +1,42 @@
 from django.db.models import Q, ProtectedError, Prefetch
 from django.contrib.auth import authenticate
+from django.conf import settings
+from django.db import IntegrityError
 
 from rest_framework import viewsets, mixins, filters, status
 from rest_framework.response import Response
-from rest_framework.permissions import AllowAny, IsAuthenticated
-from rest_framework.decorators import detail_route, api_view, authentication_classes, permission_classes
+from rest_framework.permissions import (
+    AllowAny, IsAuthenticated, DjangoModelPermissions)
+from rest_framework.decorators import (
+    action, api_view, authentication_classes, permission_classes)
 from rest_framework.generics import get_object_or_404
-from rest_framework.exceptions import APIException, NotFound
+from rest_framework.exceptions import APIException, NotFound, ValidationError
 from rest_framework.authtoken.models import Token
 
-from dispatch.modules.integrations.integrations import integrationLib, IntegrationNotFound, IntegrationCallbackError
+from dispatch.modules.integrations.integrations import (
+    integrationLib, IntegrationNotFound, IntegrationCallbackError)
 from dispatch.modules.actions.actions import list_actions, recent_articles
 
 from dispatch.models import (
     Article, File, Image, ImageAttachment, ImageGallery, Issue,
-    Page, Author, Person, Section, Tag, Topic, User, Video)
+    Page, Author, Person, Section, Tag, Topic, User, Video,
+    Poll, PollAnswer, PollVote, Invite, Subsection)
+from dispatch.modules.podcasts.models import Podcast, PodcastEpisode
+
+from dispatch.core.settings import get_settings
+from dispatch.admin.registration import reset_password
 
 from dispatch.api.mixins import DispatchModelViewSet, DispatchPublishableMixin
 from dispatch.api.serializers import (
-    ArticleSerializer, PageSerializer, SectionSerializer, ImageSerializer, FileSerializer, IssueSerializer,
-    ImageGallerySerializer, TagSerializer, TopicSerializer, PersonSerializer, UserSerializer,
-    IntegrationSerializer, ZoneSerializer, WidgetSerializer, TemplateSerializer, VideoSerializer)
-from dispatch.api.exceptions import ProtectedResourceError, BadCredentials
+    ArticleSerializer, PageSerializer, SectionSerializer, ImageSerializer,
+    FileSerializer, IssueSerializer, ImageGallerySerializer, TagSerializer,
+    TopicSerializer, PersonSerializer, UserSerializer, IntegrationSerializer,
+    ZoneSerializer, WidgetSerializer, TemplateSerializer, VideoSerializer,
+    PollSerializer, PollVoteSerializer, InviteSerializer, SubsectionSerializer,
+    PodcastSerializer, PodcastEpisodeSerializer)
+from dispatch.api.exceptions import (
+    ProtectedResourceError, BadCredentials, PollClosed, InvalidPoll,
+    UnpermittedActionError)
 
 from dispatch.theme import ThemeManager
 from dispatch.theme.exceptions import ZoneNotFound, TemplateNotFound
@@ -39,19 +54,6 @@ class SectionViewSet(DispatchModelViewSet):
             queryset = queryset.filter(name__icontains=q)
         return queryset
 
-class VideoViewSet(DispatchModelViewSet):
-    """Viewset for Video model views."""
-    model = Video
-    serializer_class = VideoSerializer
-
-    def get_queryset(self):
-        queryset = Video.objects.all()
-        q = self.request.query_params.get('q', None)
-        if q is not None:
-            # If a search term (q) is present, filter queryset by term against `title`
-            queryset = queryset.filter(title__icontains=q)
-        return queryset
-
 class ArticleViewSet(DispatchModelViewSet, DispatchPublishableMixin):
     """Viewset for Article model views."""
     model = Article
@@ -67,17 +69,18 @@ class ArticleViewSet(DispatchModelViewSet, DispatchPublishableMixin):
 
         # Optimize queries by prefetching related data
         queryset = queryset \
-            .select_related('featured_image', 'topic', 'section') \
+            .select_related('featured_image', 'featured_video', 'topic', 'section', 'subsection') \
             .prefetch_related(
                 'tags',
                 'featured_image__image__authors',
                 'authors'
             )
 
-        queryset = queryset.order_by('-updated_at') \
+        queryset = queryset.order_by('-updated_at')
 
         q = self.request.query_params.get('q', None)
         section = self.request.query_params.get('section', None)
+        tags = self.request.query_params.getlist('tags', None)
         author = self.request.query_params.get('author', None)
 
         if q is not None:
@@ -86,11 +89,33 @@ class ArticleViewSet(DispatchModelViewSet, DispatchPublishableMixin):
         if section is not None:
             queryset = queryset.filter(section_id=section)
 
+        if tags is not None:
+            for tag in tags:
+                queryset = queryset.filter(tags__id=tag)
+
         if author is not None:
             queryset = queryset.filter(authors__person_id=author)
 
         return queryset
 
+class SubsectionViewSet(DispatchModelViewSet):
+    """Viewset for the Subsection model views."""
+    model = Subsection
+    serializer_class = SubsectionSerializer
+
+    def get_queryset(self):
+        queryset = Subsection.objects.all()
+        q = self.request.query_params.get('q', None)
+        section = self.request.query_params.get('section', None)
+
+        if q is not None:
+            queryset = queryset.filter(name__icontains=q)
+
+        if section is not None:
+            queryset = queryset.filter(section_id=section)
+
+        return queryset
+
 class PageViewSet(DispatchModelViewSet, DispatchPublishableMixin):
     """Viewset for Page model views."""
     model = Page
@@ -133,7 +158,52 @@ class PersonViewSet(DispatchModelViewSet):
         try:
             instance.delete()
         except ProtectedError:
-            raise ProtectedResourceError('Deletion failed because person belongs to a user')
+            raise ProtectedResourceError(
+                'Deletion failed because person belongs to a user'
+            )
+
+    @action(detail=True, methods=['get'])
+    def user(self, request, pk=None):
+        queryset = Person.objects.all()
+
+        person = get_object_or_404(queryset, pk=pk)
+
+        try:
+            user = User.objects.get(person=person)
+            serializer = UserSerializer(user)
+        except User.DoesNotExist:
+            return Response()
+
+        return Response(serializer.data)
+
+    @action(detail=True, methods=['get'])
+    def invite(self, request, pk=None):
+        queryset = Person.objects.all()
+
+        person = get_object_or_404(queryset, pk=pk)
+
+        try:
+            invite = Invite.objects.get(person=person)
+            serializer = InviteSerializer(invite)
+        except Invite.DoesNotExist:
+            return Response({'detail': 'Person has no invitation'})
+
+        return Response(serializer.data)
+
+
+class InviteViewSet(DispatchModelViewSet):
+    """Viewset for the Invite model views."""
+    model = Invite
+    serializer_class = InviteSerializer
+
+    permission_classes = (DjangoModelPermissions,)
+
+    def get_queryset(self):
+        queryset = Invite.objects.all()
+        q = self.request.query_params.get('q', None)
+        if q is not None:
+            queryset = queryset.filter(person__id=q)
+        return queryset
 
 class UserViewSet(DispatchModelViewSet):
     """Viewset for User model views."""
@@ -142,29 +212,52 @@ class UserViewSet(DispatchModelViewSet):
 
     queryset = User.objects.all()
 
-    permission_classes = (IsAuthenticated,)
+    def get_permissions(self):
+        if self.request.method == 'PATCH':
+            self.permission_classes = [IsAuthenticated, ]
+        else:
+            self.permission_classes = [DjangoModelPermissions, ]
+        return super(UserViewSet, self).get_permissions()
+
+    def get_queryset(self):
+        queryset = User.objects.all()
+        q = self.request.query_params.get('q', None)
+
+        if q is not None:
+            queryset = queryset.filter(person__id=q)
+        return queryset
 
     def retrieve(self, request, pk=None):
         queryset = User.objects.all()
+
         if pk == 'me':
             pk = request.user.id
+
         user = get_object_or_404(queryset, pk=pk)
         serializer = UserSerializer(user)
+
         return Response(serializer.data)
 
-class TagViewSet(DispatchModelViewSet):
-    """Viewset for Tag model views."""
-    model = Tag
-    serializer_class = TagSerializer
+    def partial_update(self, request, pk=None):
+        user = User.objects.get(pk=pk)
 
-    def get_queryset(self):
-        queryset = Tag.objects.all()
-        q = self.request.query_params.get('q', None)
-        if q is not None:
-            # If a search term (q) is present, filter queryset by term against `name`
-            queryset = queryset.filter(name__icontains=q)
-        
-        return queryset
+        if user != request.user and not request.user.has_perm('dispatch.change_user'):
+            raise UnpermittedActionError()
+
+        permissions = request.data.get('permissions', None)
+        user.modify_permissions(permissions)
+
+        return super(UserViewSet, self).partial_update(request)
+
+    @action(detail=True, methods=['post'])
+    def reset_password(self, request, pk=None):
+        user = get_object_or_404(User.objects.all(), pk=pk)
+
+        if request.user.has_perm('dispatch.change_user'):
+            reset_password(user.email, request)
+            return Response(status.HTTP_202_ACCEPTED)
+        else:
+            raise UnpermittedActionError()
 
 class TopicViewSet(DispatchModelViewSet):
     """Viewset for Topic model views."""
@@ -205,6 +298,46 @@ class IssueViewSet(DispatchModelViewSet):
             queryset = queryset.filter(title__icontains=q)
         return queryset
 
+class TagViewSet(DispatchModelViewSet):
+    """Viewset for Tag model views."""
+    model = Tag
+    serializer_class = TagSerializer
+
+    def get_queryset(self):
+        queryset = Tag.objects.all()
+        q = self.request.query_params.get('q', None)
+        if q is not None:
+            # If a search term (q) is present, filter queryset by term against `name`
+            queryset = queryset.filter(name__icontains=q)
+
+        return queryset
+
+class VideoViewSet(DispatchModelViewSet):
+    """Viewset for Video model views."""
+    model = Video
+    serializer_class = VideoSerializer
+    filter_backends = (filters.OrderingFilter,)
+    update_fields = ('title', 'authors', 'tags')
+
+    def get_queryset(self):
+        queryset = Video.objects.order_by('-updated_at')
+        author = self.request.query_params.get('author', None)
+        tags = self.request.query_params.getlist('tags', None)
+        q = self.request.query_params.get('q', None)
+
+        if author is not None:
+            queryset = queryset.filter(authors__person_id=author)
+
+        if tags is not None:
+            for tag in tags:
+                queryset = queryset.filter(tags__id=tag)
+
+        if q is not None:
+            # If a search term (q) is present, filter queryset by term against `title`
+            queryset = queryset.filter(title__icontains=q)
+            
+        return queryset
+
 class ImageViewSet(viewsets.ModelViewSet):
     """Viewset for Image model views."""
     model = Image
@@ -214,7 +347,7 @@ class ImageViewSet(viewsets.ModelViewSet):
     update_fields = ('title', 'authors', 'tags')
 
     def get_queryset(self):
-        queryset = Image.objects.all()
+        queryset = Image.objects.order_by('-updated_at')
 
         author = self.request.query_params.get('author', None)
         tags = self.request.query_params.getlist('tags', None)
@@ -222,12 +355,13 @@ class ImageViewSet(viewsets.ModelViewSet):
 
         if author is not None:
             queryset = queryset.filter(authors__person_id=author)
+
         if tags is not None:
             for tag in tags:
                 queryset = queryset.filter(tags__id=tag)
 
         if q is not None:
-            queryset = queryset.filter(Q(title__icontains=q) | Q(img__icontains=q) )
+            queryset = queryset.filter(Q(title__icontains=q) | Q(img__icontains=q))
 
         return queryset
 
@@ -248,8 +382,46 @@ class ImageGalleryViewSet(DispatchModelViewSet):
             queryset = queryset.filter(title__icontains=q)
         return queryset
 
+class PollViewSet(DispatchModelViewSet):
+    """Viewset for the Poll model views."""
+    model = Poll
+    serializer_class = PollSerializer
+
+    def get_queryset(self):
+        queryset = Poll.objects.all()
+        q = self.request.query_params.get('q', None)
+        if q is not None:
+            queryset = queryset.filter(Q(name__icontains=q) | Q(question__icontains=q) )
+        return queryset
+
+    @action(detail=True, permission_classes=[AllowAny], methods=['post'],)
+    def vote(self, request, pk=None):
+        poll = get_object_or_404(Poll.objects.all(), pk=pk)
+
+        if not poll.is_open:
+            raise PollClosed()
+
+        answer = get_object_or_404(PollAnswer.objects.all(), pk=request.data['answer_id'])
+
+        if answer.poll != poll:
+            raise InvalidPoll()
+
+        # Change vote
+        if 'vote_id' in request.data:
+            vote_id = request.data['vote_id']
+            vote = PollVote.objects.filter(answer__poll=poll, id=vote_id) \
+                .update(answer=answer)
+            return Response({'id': vote_id})
+
+        serializer = PollVoteSerializer(data=request.data)
+
+        serializer.is_valid(raise_exception=True)
+        serializer.save()
+
+        return Response(serializer.data)
+
 class TemplateViewSet(viewsets.GenericViewSet):
-    """Viewset for Template views"""
+    """Viewset for Template views."""
     permission_classes = (IsAuthenticated,)
 
     def get_object_or_404(self, pk=None):
@@ -265,7 +437,14 @@ class TemplateViewSet(viewsets.GenericViewSet):
         })
 
     def list(self, request):
-        templates = ThemeManager.Templates.list()
+        q = self.request.query_params.get('q', None)
+
+        if q is not None:
+             templates = [x for x in ThemeManager.Templates.list() if x.name.lower().__contains__(q.lower())]
+
+        else:
+            templates = ThemeManager.Templates.list()
+
         serializer = TemplateSerializer(templates, many=True)
         return self.get_paginated_response(serializer.data)
 
@@ -318,19 +497,19 @@ class IntegrationViewSet(viewsets.GenericViewSet):
 
         return Response(status=status.HTTP_204_NO_CONTENT)
 
-    @detail_route(methods=['get'],)
+    @action(detail=True, methods=['get'],)
     def callback(self, request, pk=None):
         integration = self.get_object_or_404(pk)
 
         try:
             data = integration.callback(request.user, request.GET)
-        except IntegrationCallbackError, e:
-            return Response({ 'detail': e.message}, status.HTTP_400_BAD_REQUEST)
+        except IntegrationCallbackError as e:
+            return Response({ 'detail': e.message }, status.HTTP_400_BAD_REQUEST)
 
         return Response(data)
 
 class ZoneViewSet(viewsets.GenericViewSet):
-    """Viewset for widget zones"""
+    """Viewset for widget zones."""
 
     permission_classes = (IsAuthenticated,)
 
@@ -373,7 +552,7 @@ class ZoneViewSet(viewsets.GenericViewSet):
 
         return Response(serializer.data)
 
-    @detail_route(methods=['get'])
+    @action(detail=True, methods=['get'])
     def widgets(self, request, pk=None):
         zone = self.get_object_or_404(pk)
 
@@ -398,7 +577,7 @@ class DashboardViewSet(viewsets.GenericViewSet):
     def list_recent_articles(self, request):
         recent = recent_articles(request.user)
 
-        articles = map(lambda a: self.get_serializer(a).data, recent)
+        articles = [self.get_serializer(a).data for a in recent]
 
         data = {
             'results': articles
@@ -412,13 +591,16 @@ class TokenViewSet(viewsets.ViewSet):
         email = request.data.get('email', None)
         password = request.data.get('password', None)
 
-        user = authenticate(username=email, password=password)
+        user = authenticate(request, username=email, password=password)
 
         if user is not None and user.is_active:
             (token, created) = Token.objects.get_or_create(user=user)
 
+            settings = get_settings(token)
+
             data = {
-                'token': unicode(token)
+                'token': str(token),
+                'settings': settings
             }
 
             return Response(data, status=status.HTTP_202_ACCEPTED)
@@ -431,7 +613,12 @@ class TokenViewSet(viewsets.ViewSet):
         except Token.DoesNotExist:
             return Response({'token_valid': False}, status=status.HTTP_404_NOT_FOUND)
 
-        return Response({'token_valid': True})
+        settings = get_settings(token)
+
+        return Response({
+            'token_valid': True,
+            'settings': settings
+        })
 
     def delete(self, request):
         token = get_object_or_404(Token, user=request.user)
@@ -439,3 +626,36 @@ class TokenViewSet(viewsets.ViewSet):
         token.delete()
 
         return Response(status=status.HTTP_204_NO_CONTENT)
+
+class PodcastViewSet(DispatchModelViewSet):
+    """Viewset for Podcast model views."""
+    model = Podcast
+    serializer_class = PodcastSerializer
+
+    def get_queryset(self):
+        queryset = Podcast.objects.all()
+        q = self.request.query_params.get('q', None)
+        if q is not None:
+            # If a search term (q) is present, filter queryset by term against `title`
+            queryset = queryset.filter(title__icontains=q)
+        return queryset
+
+class PodcastEpisodeViewSet(DispatchModelViewSet):
+    """Viewset for PodcastEpisode model views."""
+    model = PodcastEpisode
+    serializer_class = PodcastEpisodeSerializer
+
+    def get_queryset(self):
+        queryset = PodcastEpisode.objects.all()
+
+        q = self.request.query_params.get('q', None)
+        podcast = self.request.query_params.get('podcast', None)
+
+        if q is not None:
+            # If a search term (q) is present, filter queryset by term against `title`
+            queryset = queryset.filter(title__icontains=q)
+
+        if podcast is not None:
+            queryset = queryset.filter(podcast_id=podcast)
+
+        return queryset
