@@ -1,16 +1,20 @@
-import StringIO
+import io
 import os
 import re
 import uuid
+import datetime
 
+from sys import getsizeof
 from jsonfield import JSONField
 from PIL import Image as Img
 
 from django.db import IntegrityError
+from django.db import transaction
+
 from django.db.models import (
     Model, DateTimeField, CharField, TextField, PositiveIntegerField,
-    ImageField, FileField, BooleanField, UUIDField, ForeignKey,
-    ManyToManyField, SlugField, SET_NULL)
+    ImageField, FileField, BooleanField, NullBooleanField, UUIDField, 
+    ForeignKey, ManyToManyField, SlugField, SET_NULL, CASCADE, F)
 from django.conf import settings
 from django.core.validators import MaxValueValidator
 from django.utils import timezone
@@ -22,25 +26,38 @@ from django.dispatch import receiver
 from dispatch.modules.content.managers import PublishableManager
 from dispatch.modules.content.render import content_to_html
 from dispatch.modules.content.mixins import AuthorMixin
-from dispatch.modules.auth.models import Person, User
+from dispatch.modules.auth.models import Person
 
 class Tag(Model):
     name = CharField(max_length=255, unique=True)
 
 class Topic(Model):
     name = CharField(max_length=255)
+    slug = SlugField(unique=True, max_length=255)
     last_used = DateTimeField(null=True)
 
     def update_timestamp(self):
         self.last_used = timezone.now()
         self.save()
 
+    def _generate_slug(self):
+        if self.name:
+            return self.name.lower().replace(' ', '-')
+
+        return None
+
+    def save(self, *args, **kwargs):
+        if not self.slug:
+            self.slug = self._generate_slug()
+        
+        super(Topic, self).save(*args, **kwargs)
+
 class Section(Model):
     name = CharField(max_length=100, unique=True)
     slug = SlugField(unique=True)
 
 class Author(Model):
-    person = ForeignKey(Person)
+    person = ForeignKey(Person, on_delete=CASCADE)
     order = PositiveIntegerField()
     type = CharField(blank=True, default='author', max_length=100)
 
@@ -54,10 +71,10 @@ class Publishable(Model):
 
     preview_id = UUIDField(default=uuid.uuid4)
     revision_id = PositiveIntegerField(default=0, db_index=True)
-    head = BooleanField(default=False, db_index=True)
+    head = NullBooleanField(default=None, db_index=True, null=True)
 
-    is_published = BooleanField(default=False, db_index=True)
-    is_active = BooleanField(default=True)
+    is_published = NullBooleanField(default=None, db_index=True)
+    is_active = NullBooleanField(default=True)
 
     published_version = PositiveIntegerField(null=True)
     latest_version = PositiveIntegerField(null=True)
@@ -68,6 +85,7 @@ class Publishable(Model):
     views = PositiveIntegerField(default=0)
 
     featured_image = ForeignKey('ImageAttachment', on_delete=SET_NULL, related_name='%(class)s_featured_image', blank=True, null=True)
+    featured_video = ForeignKey('VideoAttachment', on_delete=SET_NULL, related_name='%(class)s_featured_video', blank=True, null=True)
 
     template = CharField(max_length=255, default='default')
     template_data = JSONField(default={})
@@ -119,14 +137,14 @@ class Publishable(Model):
     @property
     def html(self):
         """Return HTML representation of content"""
-        return content_to_html(self.content)
+        return content_to_html(self.content, self.id)
 
     def is_parent(self):
         return self.parent is None
 
     def publish(self):
         # Unpublish last published version
-        type(self).objects.filter(parent=self.parent, is_published=True).update(is_published=False, published_at=None)
+        type(self).objects.filter(parent=self.parent, is_published=True).update(is_published=None, published_at=None)
         self.is_published = True
         if self.published_at is None:
             self.published_at = timezone.now()
@@ -139,8 +157,8 @@ class Publishable(Model):
         return self
 
     def unpublish(self):
-        type(self).objects.filter(parent=self.parent, is_published=True).update(is_published=False, published_at=None)
-        self.is_published = False
+        type(self).objects.filter(parent=self.parent, is_published=True).update(is_published=None, published_at=None)
+        self.is_published = None
 
         # Unset published version for all articles
         type(self).objects.filter(parent=self.parent).update(published_version=None)
@@ -149,6 +167,7 @@ class Publishable(Model):
         return self
 
     # Overriding
+    @transaction.atomic
     def save(self, revision=True, *args, **kwargs):
         """
         Handles the saving/updating of a Publishable instance.
@@ -166,7 +185,9 @@ class Publishable(Model):
 
             if not self.is_parent():
                 # If this is a revision, delete the old head of the list.
-                type(self).objects.filter(parent=self.parent, head=True).update(head=False)
+                type(self).objects \
+                    .filter(parent=self.parent, head=True) \
+                    .update(head=None)
 
                 # Clear the instance id to force Django to save a new instance.
                 # Both fields (pk, id) required for this to work -- something to do with model inheritance
@@ -174,11 +195,7 @@ class Publishable(Model):
                 self.id = None
 
                 # New version is unpublished by default
-                self.is_published = False
-
-        # Raise integrity error if instance with given slug already exists.
-        if type(self).objects.filter(slug=self.slug).exclude(parent=self.parent).exists():
-            raise IntegrityError("%s with slug '%s' already exists." % (type(self).__name__, self.slug))
+                self.is_published = None
 
         # Set created_at to current time, but only for first version
         if not self.created_at:
@@ -197,11 +214,21 @@ class Publishable(Model):
 
         if revision:
             # Set latest version for all articles
-            type(self).objects.filter(parent=self.parent).update(latest_version=self.revision_id)
+            type(self).objects \
+                .filter(parent=self.parent) \
+                .update(latest_version=self.revision_id)
+                
             self.latest_version = self.revision_id
 
         return self
 
+    # Overriding delete the parent article to cascade delete all versions
+    def delete(self, *args, **kwargs):
+        if self.parent == self:
+            return super(Publishable, self).delete(*args, **kwargs)
+        
+        return self.parent.delete()
+
     def save_featured_image(self, data):
         """
         Handles saving the featured image.
@@ -221,6 +248,15 @@ class Publishable(Model):
         if data is None:
             if attachment:
                 attachment.delete()
+
+            self.featured_image = None
+            return
+
+        if data['image_id'] is None:
+            if attachment:
+                attachment.delete()
+
+            self.featured_image = None
             return
 
         if not attachment:
@@ -238,11 +274,45 @@ class Publishable(Model):
 
         self.featured_image = attachment
 
+    def save_featured_video(self, data):
+        attachment = self.featured_video
+
+        if data is None:
+            if attachment:
+                attachment.delete()
+
+            self.featured_video = None
+            return
+
+        if data['video_id'] is None:
+            if attachment:
+                attachment.delete()
+
+            self.featured_video = None
+            return
+
+        if not attachment:
+            attachment = VideoAttachment()
+
+        attachment.video_id = data.get('video_id', attachment.video_id)
+        attachment.caption = data.get('caption', None)
+        attachment.credit = data.get('credit', None)
+
+        instance_type = str(type(self)).lower()
+
+        setattr(attachment, instance_type, self)
+
+        attachment.save()
+
+        self.featured_video = attachment
+
     def get_previous_revision(self):
         if self.parent == self:
             return self
         try:
-            revision = type(self).objects.filter(parent=self.parent).order_by('-pk')[1]
+            revision = type(self).objects \
+                .filter(parent=self.parent) \
+                .order_by('-pk')[1]
             return revision
         except:
             return self
@@ -251,15 +321,18 @@ class Publishable(Model):
         abstract = True
 
 class Article(Publishable, AuthorMixin):
-
-    parent = ForeignKey('Article', related_name='article_parent', blank=True, null=True)
+    parent = ForeignKey('Article', on_delete=SET_NULL, related_name='article_parent', blank=True, null=True)
 
     headline = CharField(max_length=255)
-    section = ForeignKey('Section')
+    section = ForeignKey('Section', on_delete=CASCADE)
+    subsection = ForeignKey('Subsection', on_delete=SET_NULL,related_name='article_subsection', blank=True, null=True)
     authors = ManyToManyField('Author', related_name='article_authors')
-    topic = ForeignKey('Topic', null=True)
+    topic = ForeignKey('Topic', on_delete=SET_NULL, null=True)
     tags = ManyToManyField('Tag')
 
+    is_breaking = BooleanField(default=False)
+    breaking_timeout = DateTimeField(blank=True, null=True)
+
     IMPORTANCE_CHOICES = [(i,i) for i in range(1,6)]
 
     importance = PositiveIntegerField(validators=[MaxValueValidator(5)], choices=IMPORTANCE_CHOICES, default=3)
@@ -273,6 +346,13 @@ class Article(Publishable, AuthorMixin):
 
     reading_time = CharField(max_length=100, choices=READING_CHOICES, default='anytime')
 
+    class Meta:
+        unique_together = (
+            ('slug', 'head'),
+            ('parent', 'slug', 'head'),
+            ('parent', 'slug', 'is_published'),
+        )
+
     AuthorModel = Author
 
     @property
@@ -291,6 +371,12 @@ class Article(Publishable, AuthorMixin):
             'name': name
         }
 
+    def is_currently_breaking(self):
+        if self.is_published and self.is_breaking:
+            if self.breaking_timeout:
+                return timezone.now() < self.breaking_timeout
+        return False
+
     def save_tags(self, tag_ids):
         self.tags.clear()
         for tag_id in tag_ids:
@@ -312,29 +398,81 @@ class Article(Publishable, AuthorMixin):
                 pass
 
     def get_absolute_url(self):
-        """
-        Returns article URL.
-        """
+        """ Returns article URL. """
         return "%s%s/%s/" % (settings.BASE_URL, self.section.slug, self.slug)
 
+    def get_subsection(self):
+        """ Returns the subsection set in the parent article """
+        return self.parent.subsection
+
+    def save_subsection(self, subsection_id):
+        """ Save the subsection to the parent article """
+        Article.objects.filter(parent_id=self.parent.id).update(subsection_id=subsection_id)
+
+class Subsection(Model, AuthorMixin):
+    name = CharField(max_length=100, unique=True)
+    slug = SlugField(unique=True)
+    description = TextField(null=True, blank=True)
+    authors = ManyToManyField('Author', related_name='subsection_authors')
+    section = ForeignKey('Section', on_delete=CASCADE)
+    is_active = BooleanField(default=False)
+
+    AuthorModel = Author
+
+    def save_articles(self, article_ids):
+        Article.objects.filter(subsection=self).update(subsection=None)
+        Article.objects.filter(parent_id__in=article_ids).update(subsection=self)
+
+    def get_articles(self):
+        return Article.objects.filter(subsection=self, id=F('parent_id'))
+
+    def get_published_articles(self):
+        return Article.objects.filter(subsection=self, is_published=True).order_by('-published_at')
+
+    def get_absolute_url(self):
+        """Returns the subsection URL."""
+        return "%s%s/" % (settings.BASE_URL, self.slug)
+
 class Page(Publishable):
-    parent = ForeignKey('Page', related_name='page_parent', blank=True, null=True)
-    parent_page = ForeignKey('Page', related_name='parent_page_fk', null=True)
+    parent = ForeignKey('Page', on_delete=SET_NULL, related_name='page_parent', blank=True, null=True)
+    parent_page = ForeignKey('Page', on_delete=SET_NULL, related_name='parent_page_fk', null=True)
     title = CharField(max_length=255)
 
+    class Meta:
+        unique_together = (
+            ('slug', 'head'),
+            ('parent', 'slug', 'head'),
+            ('parent', 'slug', 'is_published'),
+        )
+
     def get_author_string(self):
         return None
 
     def get_absolute_url(self):
-        """
-        Returns page URL.
-        """
+        """ Returns page URL. """
         return "%s%s/" % (settings.BASE_URL, self.slug)
 
-class Video(Model):
+class Video(Model, AuthorMixin):
     title = CharField(max_length=255)
     url = CharField(max_length=500)
 
+    authors = ManyToManyField(Author, related_name='video_authors')
+    tags = ManyToManyField('Tag')
+
+    created_at = DateTimeField(default=timezone.now)
+    updated_at = DateTimeField(auto_now=True)
+
+    AuthorModel = Author
+    
+    def save_tags(self, tag_ids):
+        self.tags.clear()
+        for tag_id in tag_ids:
+            try:
+                tag = Tag.objects.get(id=int(tag_id))
+                self.tags.add(tag)
+            except Tag.DoesNotExist:
+                pass
+
 class Image(Model, AuthorMixin):
     img = ImageField(upload_to='images/%Y/%m')
     title = CharField(max_length=255, blank=True, null=True)
@@ -400,7 +538,7 @@ class Image(Model, AuthorMixin):
     def save(self, **kwargs):
         """Custom save method to process thumbnails and save image dimensions."""
         is_new = self.pk is None
-
+        
         if is_new:
             # Make filenames lowercase
             self.img.name = self.img.name.lower()
@@ -409,7 +547,13 @@ class Image(Model, AuthorMixin):
         super(Image, self).save(**kwargs)
 
         if is_new and self.img:
-            image = Img.open(StringIO.StringIO(self.img.read()))
+            data = self.img.read()
+
+            if not data:
+                return
+
+            image = Img.open(io.BytesIO(data))
+
             self.width, self.height = image.size
 
             super(Image, self).save()
@@ -417,7 +561,7 @@ class Image(Model, AuthorMixin):
             name = self.get_name()
             ext = self.get_extension()
 
-            for size in self.SIZES.keys():
+            for size in list(self.SIZES.keys()):
                 self.save_thumbnail(image, self.SIZES[size], name, size, ext)
 
     def save_thumbnail(self, image, size, name, label, file_type):
@@ -427,7 +571,7 @@ class Image(Model, AuthorMixin):
 
         # If image is larger than thumbnail size, resize image
         if (imw > width) or (imh > height):
-            image.thumbnail(size, Img.ANTIALIAS)
+            image.thumbnail(size, Img.LANCZOS)
 
         # Attach new thumbnail label to image filename
         name = "%s-%s.jpg" % (name, label)
@@ -437,16 +581,15 @@ class Image(Model, AuthorMixin):
             file_type = 'JPEG'
 
         # Write new thumbnail to StringIO object
-        image_io = StringIO.StringIO()
+        image_io = io.BytesIO()
         image.save(image_io, format=file_type, quality=75)
 
         # Convert StringIO object to Django File object
-        thumb_file = InMemoryUploadedFile(image_io, None, name, 'image/jpeg', image_io.len, None)
+        thumb_file = InMemoryUploadedFile(image_io, None, name, 'image/jpeg', getsizeof(image_io), None)
 
         # Save the new file to the default storage system
         default_storage.save(name, thumb_file)
 
-
     def save_tags(self, tag_ids):
         self.tags.clear()
         for tag_id in tag_ids:
@@ -456,13 +599,25 @@ class Image(Model, AuthorMixin):
             except Tag.DoesNotExist:
                 pass
 
-class ImageAttachment(Model):
-    article = ForeignKey(Article, blank=True, null=True, related_name='article')
-    page = ForeignKey(Page, blank=True, null=True, related_name='page')
-    gallery = ForeignKey('ImageGallery', blank=True, null=True)
+class VideoAttachment(Model):
+    article = ForeignKey(Article, on_delete=SET_NULL, blank=True, null=True, related_name='video_article')
+    page = ForeignKey(Page, on_delete=SET_NULL, blank=True, null=True, related_name='video_page')
 
     caption = TextField(blank=True, null=True)
     credit = TextField(blank=True, null=True)
+    video = ForeignKey(Video, related_name='video', on_delete=SET_NULL, null=True)
+
+    order = PositiveIntegerField(null=True)
+
+class ImageAttachment(Model):
+    article = ForeignKey(Article, on_delete=SET_NULL, blank=True, null=True, related_name='image_article')
+    page = ForeignKey(Page, on_delete=SET_NULL, blank=True, null=True, related_name='image_page')
+    gallery = ForeignKey('ImageGallery', on_delete=SET_NULL, blank=True, null=True)
+
+    caption = TextField(blank=True, null=True)
+    credit = TextField(blank=True, null=True)
+    style = CharField(max_length=255, blank=True, null=True)
+    width = CharField(max_length=255, blank=True, null=True)
     image = ForeignKey(Image, related_name='image', on_delete=SET_NULL, null=True)
 
     order = PositiveIntegerField(null=True)
@@ -502,3 +657,43 @@ class Issue(Model):
     volume = PositiveIntegerField(null=True)
     issue = PositiveIntegerField(null=True)
     date = DateTimeField()
+
+class Poll(Model):
+    name = CharField(max_length=255)
+    question = CharField(max_length=255)
+    is_open = BooleanField(default=True)
+    show_results = BooleanField(default=True)
+
+    @transaction.atomic
+    def save_answers(self, answers, is_new):
+        if not is_new:
+            self.delete_old_answers(answers)
+        for answer in answers:
+            try:
+                answer_id = answer.get('id')
+                answer_obj = PollAnswer.objects.get(poll=self, id=answer_id)
+                answer_obj.name = answer['name']
+            except PollAnswer.DoesNotExist:
+                answer_obj = PollAnswer(poll=self, name=answer['name'])
+            answer_obj.save()
+
+    def delete_old_answers(self, answers):
+        PollAnswer.objects.filter(poll=self) \
+            .exclude(id__in=[answer.get('id', 0) for answer in answers]) \
+            .delete()
+
+    def get_total_votes(self):
+        return PollVote.objects.filter(answer__poll=self).count()
+
+class PollAnswer(Model):
+    poll = ForeignKey(Poll, related_name='answers', on_delete=CASCADE)
+    name = CharField(max_length=255)
+
+    def get_vote_count(self):
+        """Return the number of votes for this answer"""
+        return PollVote.objects.filter(answer=self).count()
+
+class PollVote(Model):
+    id = UUIDField(default=uuid.uuid4, primary_key=True)
+    answer = ForeignKey(PollAnswer, related_name='votes', on_delete=CASCADE)
+    timestamp = DateTimeField(auto_now_add=True)
