@@ -1,15 +1,15 @@
 import datetime
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 
 from rest_framework import status
 
-from dispatch.tests.cases import DispatchAPITestCase
+from dispatch.tests.cases import DispatchAPITestCase, DispatchMediaTestMixin
 from dispatch.tests.helpers import DispatchTestHelpers
 from dispatch.models import Article, Author, Person, Section
 from dispatch.modules.content.mixins import AuthorMixin
 from django.utils import timezone
 
-class ArticlesTests(DispatchAPITestCase):
+class ArticlesTests(DispatchAPITestCase, DispatchMediaTestMixin):
 
     def test_create_article_unauthorized(self):
         """Create article should fail with unauthenticated request"""
@@ -61,7 +61,7 @@ class ArticlesTests(DispatchAPITestCase):
         # Check data
         self.assertEqual(response.data['headline'], 'Test headline')
         self.assertEqual(response.data['section']['name'], 'Test Section')
-        self.assertEqual(response.data['authors'][0]['person']['full_name'], 'Test Person')
+        self.assertEqual(response.data['authors'][0]['person']['full_name'], 'The Test Person')
         self.assertEqual(response.data['slug'], 'test-article')
 
     def test_create_article_existing_slug(self):
@@ -162,10 +162,10 @@ class ArticlesTests(DispatchAPITestCase):
 
         url = reverse('api-articles-list')
 
-        (person1, created) = Person.objects.get_or_create(full_name='Test Person')
-        (person2, created) = Person.objects.get_or_create(full_name='Test Person 2')
-        (person3, created) = Person.objects.get_or_create(full_name='Test Person 3')
-        (person4, created) = Person.objects.get_or_create(full_name='Test Person 4')
+        (person1, created) = Person.objects.get_or_create(full_name='Test Person', slug='test-person')
+        (person2, created) = Person.objects.get_or_create(full_name='Test Person 2' , slug='test-person-2')
+        (person3, created) = Person.objects.get_or_create(full_name='Test Person 3', slug='test-person-3')
+        (person4, created) = Person.objects.get_or_create(full_name='Test Person 4', slug='test-person-4')
         (section, created) = Section.objects.get_or_create(name='Test Section', slug='test-section')
 
         data = {
@@ -187,7 +187,7 @@ class ArticlesTests(DispatchAPITestCase):
 
         article_string = article.get_author_type_string()
 
-        self.assertEqual(article_string, 'Written by Test Person, Photos by Test Person 2, Illustrations by Test Person 3, Videos by Test Person 4')
+        self.assertEqual(article_string, 'Written by <a href="/authors/test-person/">Test Person</a>, Photos by <a href="/authors/test-person-2/">Test Person 2</a>, Illustrations by <a href="/authors/test-person-3/">Test Person 3</a>, Videos by <a href="/authors/test-person-4/">Test Person 4</a>')
 
     def test_author_person(self):
         """Should not be able to create article with an author type and missing author person"""
@@ -211,7 +211,7 @@ class ArticlesTests(DispatchAPITestCase):
     def test_author_type(self):
         """Should be able to create article with an author and missing author type"""
 
-        (person, created) = Person.objects.get_or_create(full_name='Test Person')
+        (person, created) = Person.objects.get_or_create(full_name='Test Person', slug='test-person')
         (section, created) = Section.objects.get_or_create(name='Test Section', slug='test-section')
 
         data = {
@@ -230,7 +230,7 @@ class ArticlesTests(DispatchAPITestCase):
     def test_author_type_format(self):
         """Should not be able to create article with an author and non-string author type"""
 
-        (person, created) = Person.objects.get_or_create(full_name='Test Person')
+        (person, created) = Person.objects.get_or_create(full_name='Test Person', slug='test-person')
         (section, created) = Section.objects.get_or_create(name='Test Section', slug='test-section')
 
         data = {
@@ -246,6 +246,10 @@ class ArticlesTests(DispatchAPITestCase):
         response = self.client.post(url, data, format='json')
         self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
+        data['author_ids'][0]['type'] = 'article author'
+        response = self.client.post(url, data, format='json')
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+
     def test_update_article_tags(self):
         """Should be able to update and remove article tags"""
 
@@ -433,6 +437,25 @@ class ArticlesTests(DispatchAPITestCase):
         response = self.client.delete(url, format='json')
         self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
 
+    def test_delete_article_multiple_versions(self):
+        """Ensure that deleting an article deletes all versions of the article"""
+
+        article = DispatchTestHelpers.create_article(self.client)
+
+        # Update the article to create a new version
+        data = {
+            'content': []
+        }
+
+        url = reverse('api-articles-detail', args=[article.data['id']])
+        response = self.client.patch(url, data, format='json')
+
+        # Delete the article
+        response = self.client.delete(url, format='json')
+
+        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
+        self.assertFalse(Article.objects.filter(parent__id=article.data['id']).exists())
+
     def test_publish_article(self):
         """Ensure that an existing article can be published"""
 
@@ -616,6 +639,25 @@ class ArticlesTests(DispatchAPITestCase):
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         self.assertEqual(response.data['count'], 1)
         self.assertEqual(response.data['results'][0]['slug'], 'article-2')
+    def test_article_exclude_query(self):
+        """Should be able to exclude fields from articles by query params"""
+
+        article_1 = DispatchTestHelpers.create_article(self.client, headline='Article 1', slug='article-1')
+        article_2 = DispatchTestHelpers.create_article(self.client, headline='Article 2', slug='article-2')
+
+        url = '%s?exclude=%s' % (reverse('api-articles-list'), 'content,section_id')
+        response = self.client.get(url, format='json')
+
+        data = response.data
+
+        self.assertNotIn('content', data['results'][0])
+        self.assertNotIn('section_id', data['results'][0])
+        self.assertIn('slug', data['results'][0])
+        self.assertNotIn('content', data['results'][1])
+        self.assertNotIn('section_id', data['results'][1])
+        self.assertIn('slug', data['results'][1])
+        self.assertEqual(data['count'], 2)
+
 
     def test_article_headline_query(self):
         """Should be able to search for articles by headline"""
@@ -678,7 +720,7 @@ class ArticlesTests(DispatchAPITestCase):
         article_1 = DispatchTestHelpers.create_article(self.client, headline='Article 1', slug='article-1')
         article_2 = DispatchTestHelpers.create_article(self.client, headline='Article 2', slug='article-2')
 
-        article_3 = DispatchTestHelpers.create_article(self.client, headline='Article 3', slug='article-3', author_names=['Test Person', 'Test Person2'])
+        article_3 = DispatchTestHelpers.create_article(self.client, headline='Article 3', slug='article-3', author_names=['The Test Person', 'Test Person2'])
 
         article_4 = DispatchTestHelpers.create_article(self.client, headline='Article 4', slug='article-2', author_names=['Test Person2'])
 
@@ -693,3 +735,147 @@ class ArticlesTests(DispatchAPITestCase):
         self.assertEqual(data['results'][1]['headline'], 'Article 2')
         self.assertEqual(data['results'][2]['headline'], 'Article 1')
         self.assertEqual(data['count'], 3)
+
+    def test_set_featured_image(self):
+        """Ensure that a featured image can be set"""
+
+        article = DispatchTestHelpers.create_article(self.client)
+        image = DispatchTestHelpers.create_image(self.client)
+
+        data = {
+            'featured_image':   {
+                'image_id': image.data['id']
+            }
+        }
+
+        url = reverse('api-articles-detail', args=[article.data['id']])
+
+        response = self.client.patch(url, data, format='json')
+
+        self.assertEqual(response.data['featured_image']['image']['id'], image.data['id'])
+
+    def test_set_featured_image_no_id(self):
+        """Ensure that there must have an image_id in order to set a featured image"""
+
+        article = DispatchTestHelpers.create_article(self.client)
+
+        data = {
+            'featured_image':   {
+                'image_id': None,
+                'credit': 'test credit',
+                'caption': 'test caption'
+            }
+        }
+
+        url = reverse('api-articles-detail', args=[article.data['id']])
+
+        response = self.client.patch(url, data, format='json')
+
+        data = {
+            'featured_image': None
+        }
+
+        response = self.client.patch(url, data, format='json')
+
+        self.assertEqual(response.data['featured_image'], None)
+
+    def test_remove_featured_image(self):
+        """Ensure that a featured image can be removed"""
+
+        article = DispatchTestHelpers.create_article(self.client)
+        image = DispatchTestHelpers.create_image(self.client)
+        
+        data = {
+            'featured_image':   {
+                'image_id': image.data['id']
+            }
+        }
+
+        url = reverse('api-articles-detail', args=[article.data['id']])
+
+        response = self.client.patch(url, data, format='json')
+
+        data = {
+            'featured_image': None
+        }
+
+        response = self.client.patch(url, data, format='json')
+
+        self.assertEqual(response.data['featured_image'], None)
+
+    def test_set_featured_video(self):
+        """Ensure that a featured video can be set"""
+
+        article = DispatchTestHelpers.create_article(self.client)
+
+        url = reverse('api-videos-list')
+
+        video = DispatchTestHelpers.create_video(self.client, 'testVideo')
+
+        data = {
+            'featured_video':   {
+                'video_id': video.data['id']
+            }
+        }
+
+        url = reverse('api-articles-detail', args=[article.data['id']])
+
+        response = self.client.patch(url, data, format='json')
+
+        self.assertEqual(response.data['featured_video']['video']['id'], video.data['id'])
+
+    def test_set_featured_video_no_id(self):
+        """Ensure that there must have a video_id in order to set a featured video"""
+
+        article = DispatchTestHelpers.create_article(self.client)
+
+        url = reverse('api-videos-list')
+
+        video = DispatchTestHelpers.create_video(self.client, 'testVideo')
+
+        data = {
+            'featured_video':   {
+                'video_id': None,
+                'credit': 'test credit',
+                'caption': 'test caption'
+            }
+        }
+
+        url = reverse('api-articles-detail', args=[article.data['id']])
+
+        response = self.client.patch(url, data, format='json')
+
+        data = {
+            'featured_video': None
+        }
+
+        response = self.client.patch(url, data, format='json')
+
+        self.assertEqual(response.data['featured_video'], None)
+
+    def test_remove_featured_video(self):
+        """Ensure that a featured video can be removed"""
+
+        article = DispatchTestHelpers.create_article(self.client)
+
+        url = reverse('api-videos-list')
+
+        video = DispatchTestHelpers.create_video(self.client, 'testVideo')
+
+        data = {
+            'featured_video':   {
+                'video_id': video.data['id']
+            }
+        }
+
+        url = reverse('api-articles-detail', args=[article.data['id']])
+
+        response = self.client.patch(url, data, format='json')
+
+        data = {
+            'featured_video': None
+        }
+
+        response = self.client.patch(url, data, format='json')
+
+        self.assertEqual(response.data['featured_video'], None)
