@@ -1,8 +1,6 @@
 from dispatch.theme.fields import MetaFields
 
-class Template:
-
-    __metaclass__ = MetaFields
+class Template(metaclass=MetaFields):
 
     def __init__(self):
         self.data = {}
